import { Injectable } from '@nestjs/common';
import { Repository } from 'typeorm';
import { Swipe } from './entities/swipe.entity';
import { InjectRepository } from '@nestjs/typeorm';
import { UsersService } from 'src/users/users.service';
import { UserIdNotExistsException } from 'src/utils/errors';
import { PairsService } from 'src/pairs/pairs.service';

@Injectable()
export class SwipesService {
  constructor(
    @InjectRepository(Swipe)
    private readonly swipesRepository: Repository<Swipe>,
    private readonly usersService: UsersService,
    private readonly pairsService: PairsService,
  ) {}

  get select() {
    return {
      id: true,
      createdAt: true,
      liked: true,
      sourceUser: { id: true, email: true, name: true },
      targetUser: { id: true, email: true, name: true },
    };
  }

  async createSwipe(
    sourceUserId: number,
    targetUserId: number,
    liked: boolean,
  ) {
    const sourceUser = await this.usersService.getById(sourceUserId);
    const targetUser = await this.usersService.getById(targetUserId);
    if (!sourceUser || !targetUser) {
      throw new UserIdNotExistsException();
    }

    const existedSwipe = await this.getSwipe(sourceUserId, targetUserId, liked);
    if (existedSwipe) {
      return existedSwipe;
    }

    const swipe = this.swipesRepository.create({
      sourceUser: { id: sourceUserId },
      targetUser: { id: targetUserId },
      liked,
    });

    await this.swipesRepository.save(swipe);

    if (liked) {
      const answeringLikeSwipe = await this.getSwipe(
        targetUserId,
        sourceUserId,
        true,
      );

      if (answeringLikeSwipe) {
        await this.pairsService.createPair([sourceUserId, targetUserId]); // TODO use transaction?
      }
    }

    return this.getSwipeById(swipe.id);
  }

  async getSwipesByUser(sourceUserId: number) {
    const user = await this.usersService.getById(sourceUserId);
    if (!user) {
      throw new UserIdNotExistsException();
    }
    return this.swipesRepository.find({
      where: { sourceUser: { id: sourceUserId } },
      select: this.select,
      relations: {
        sourceUser: true,
        targetUser: true,
      },
    });
  }

  async getSwipe(sourceUserId: number, targetUserId: number, liked: boolean) {
    return this.swipesRepository.findOne({
      where: {
        sourceUser: { id: sourceUserId },
        targetUser: { id: targetUserId },
        liked,
      },
      select: this.select,
      relations: {
        sourceUser: true,
        targetUser: true,
      },
    });
  }

  async getSwipeById(id: number) {
    return this.swipesRepository.findOne({
      where: { id },
      select: this.select,
      relations: {
        sourceUser: true,
        targetUser: true,
      },
    });
  }
}
